from math import floor

from django import template
from django.templatetags.static import static

from config.cache_durations import *
from gallery.cache import cached_or_set

register = template.Library()


@register.filter
def photo_image_data(photo):
    """
    Returns resized image data (URL, dimensions, aspect ratio) for a Photo object.
    Caches the result for PHOTO_IMAGE_DATA_DURATION.
    Returns a static placeholder if photo is missing or lacks dimensions.
    """
    if not photo:
        return {
            "url": static("img/placeholder.png"),
            "width": 1200,
            "height": 800,
            "aspect_ratio": round(1200 / 800, 3),
            "is_placeholder": True,
        }

    def generate_data():
        max_w, max_h = 720, 720

        if not photo.width or not photo.height:
            return {
                "url": static("img/placeholder.png"),
                "width": 1200,
                "height": 800,
                "aspect_ratio": round(1200 / 800, 3),
                "is_placeholder": True,
            }

        aspect = photo.aspect_ratio
        if photo.width > photo.height:
            w = min(photo.width, max_w)
            h = floor(w / aspect)
        else:
            h = min(photo.height, max_h)
            w = floor(h * aspect)

        return {
            "url": photo.photo_md.url,
            "width": w,
            "height": h,
            "aspect_ratio": round(aspect, 3),
            "is_placeholder": False,
        }

    cache_key = f'photo_md_image_data_{photo.pk}'
    return cached_or_set(cache_key, PHOTO_MD_IMAGE_DATA_DURATION, generate_data)
